import { Switch, Route, Redirect } from "wouter";
import { queryClient } from "./lib/queryClient";
import { QueryClientProvider } from "@tanstack/react-query";
import { Toaster } from "@/components/ui/toaster";
import { TooltipProvider } from "./components/ui/tooltip";
import { configureToastDuration } from "@/hooks/use-toast";
import { AuthProvider, useAuth } from "@/hooks/use-auth";
import { WaitlistProvider } from "@/hooks/use-waitlist";
import { ProfileProvider } from "@/hooks/use-profile-context";
import { ErrorBoundary } from "@/components/error-boundary";
// Temporarily removed feedback button import
// import { FeedbackButton } from "./components/feedback-button";
import { ProtectedRoute } from "./lib/protected-route";
import { WaitlistCheck } from "@/components/waitlist-check";
import NotFound from "@/pages/not-found";
import AuthPage from "@/pages/auth-page";
import HomePage from "@/pages/home-page";
import TemplatesPage from "@/pages/templates-page";
import ReviewGeneratorPage from "@/pages/review-generator-page";
import ReviewsListPage from "@/pages/reviews-list-page";
import ReviewDetailPage from "@/pages/review-detail-page";
import FeatureFlagsPage from "@/pages/admin/feature-flags";
import ResetPasswordPage from "@/pages/reset-password-page";
import ProfileSetupPage from "@/pages/profile-setup-page";
import VerifyEmailPage from "@/pages/verify-email-page";
import EmailVerificationSuccess from "@/pages/email-verification-success";
import EmailVerificationError from "@/pages/email-verification-error";
import SubscriptionPage from "@/pages/subscription-page";
import flagsmith from "flagsmith";
import { FlagsmithProvider } from "flagsmith/react";
import { useFlags } from "flagsmith/react";
import { HelmetProvider } from "react-helmet-async";
import React from 'react';
import SettingsPage from "@/pages/settings-page";
import PrivacyPolicyPage from "@/pages/privacy-policy-page";
import TermsOfServicePage from "@/pages/terms-of-service-page";
import ProjectsPage from "@/pages/projects-page";
import ProjectDetailPage from "@/pages/project-detail-page";
import ComponentTestPage from "@/pages/component-test-page";
import ToastDemoPage from "@/pages/toast-demo-page";


// Feature Flag aware route component that only renders children if flag is enabled
type FeatureFlagRouteProps = {
  flagName: string;
  children: React.ReactNode;
  fallback?: React.ReactNode;
};

// Export the component so it can be used in the Router
export function FeatureFlagRoute({ flagName, children, fallback = null }: FeatureFlagRouteProps) {
  const flags = useFlags([flagName]);
  // Flagsmith flags can be boolean values directly
  const isEnabled = typeof flags[flagName] === 'boolean'
    ? flags[flagName]
    : (flags[flagName] as any)?.enabled;

  if (!isEnabled) {
    return <>{fallback}</>;
  }

  return <>{children}</>;
}

// Note: We'll initialize external logging after auth is ready
import { initializeExternalLogging } from "./lib/external-logging";

// Wrapper component to initialize external logging with user context
function ExternalLoggingInitializer({ children }: { children: React.ReactNode }) {
  const { user } = useAuth();

  React.useEffect(() => {
    // Initialize external logging with user ID if available
    initializeExternalLogging(user?.id);
  }, [user?.id]);

  return <>{children}</>;
}

interface WithProtectedAppProps {
  children?: React.ReactNode;
}

const withProtectedApp = (WrappedComponent: React.ComponentType<WithProtectedAppProps>) => {
  return function WithProtectedAppWrapper(props: WithProtectedAppProps) {
    const { user } = useAuth();

    // If the user's email is not verified, we don't even reach the waitlist check
    // The ProtectedRoute component will handle the redirect to the verification page

    return (
      <WaitlistCheck>
        <WrappedComponent {...props} />
      </WaitlistCheck>
    );
  };
};

const ProtectedHomePage = withProtectedApp(HomePage);
const ProtectedTemplatesPage = withProtectedApp(TemplatesPage);
const ProtectedReviewGeneratorPage = withProtectedApp(ReviewGeneratorPage);
const ProtectedReviewsListPage = withProtectedApp(ReviewsListPage);
const ProtectedReviewDetailPage = withProtectedApp(ReviewDetailPage);
const ProtectedSubscriptionPage = withProtectedApp(SubscriptionPage);
const ProtectedProjectsPage = withProtectedApp(ProjectsPage);
const ProtectedProjectDetailPage = withProtectedApp(ProjectDetailPage);

function Router() {
  return (
    <Switch>
      <Route path="/auth" component={AuthPage} />
      <Route path="/reset-password/:token" component={ResetPasswordPage} />
      <Route path="/email-verification-success" component={EmailVerificationSuccess} />
      <Route path="/email-verification-error" component={EmailVerificationError} />
      <Route path="/privacy-policy" component={PrivacyPolicyPage} />
      <Route path="/terms-of-service" component={TermsOfServicePage} />
      <ProtectedRoute path="/auth/verify-email">
        <VerifyEmailPage />
      </ProtectedRoute>

      {/* Profile setup is protected but not behind waitlist */}
      <ProtectedRoute path="/profile/setup">
        <ProfileSetupPage />
      </ProtectedRoute>

      {/* Protected routes behind waitlist */}
      <ProtectedRoute path="/">
        <ProtectedHomePage />
      </ProtectedRoute>
      <ProtectedRoute path="/dashboard">
        <Redirect to="/" />
      </ProtectedRoute>
      <ProtectedRoute path="/templates">
        <ProtectedTemplatesPage />
      </ProtectedRoute>

      {/* Review routes with A/B testing */}
      <ProtectedRoute path="/reviews/:id">
        <FeatureFlagRoute
          flagName="new_review_ui"
          fallback={<Redirect to="/reviews" />}
        >
          <ProtectedReviewDetailPage />
        </FeatureFlagRoute>
      </ProtectedRoute>

      <ProtectedRoute path="/reviews">
        <FeatureFlagRoute
          flagName="new_review_ui"
          fallback={<ProtectedReviewGeneratorPage />}
        >
          <ProtectedReviewsListPage />
        </FeatureFlagRoute>
      </ProtectedRoute>

      <ProtectedRoute path="/subscription">
        <ProtectedSubscriptionPage />
      </ProtectedRoute>
      <ProtectedRoute path="/settings">
        <SettingsPage />
      </ProtectedRoute>

      {/* Project detail page - must be placed first for correct route matching */}
      <ProtectedRoute path="/projects/:id">
        <FeatureFlagRoute
          flagName="support_project_grouping"
          fallback={<Redirect to="/" />}
        >
          <ProtectedProjectDetailPage />
        </FeatureFlagRoute>
      </ProtectedRoute>

      {/* Projects main page - must be placed after detail page */}
      <ProtectedRoute path="/projects">
        <FeatureFlagRoute
          flagName="support_project_grouping"
          fallback={<Redirect to="/" />}
        >
          <ProtectedProjectsPage />
        </FeatureFlagRoute>
      </ProtectedRoute>

      <ProtectedRoute path="/admin/feature-flags">
        <FeatureFlagsPage />
      </ProtectedRoute>

      {/* Component test page for showcasing reusable components */}
      <Route path="/component-test" component={ComponentTestPage} />
      
      {/* Toast demo page for testing toast functionality */}
      <Route path="/toast-demo" component={ToastDemoPage} />
      
      <Route component={NotFound} />
    </Switch>
  );
}

function App() {
  // Set default toast duration to 5 seconds (5000ms)
  React.useEffect(() => {
    configureToastDuration(5000);
  }, []);
  
  return (
    <ErrorBoundary>
      <QueryClientProvider client={queryClient}>
        <HelmetProvider>
          <FlagsmithProvider
            options={{
              environmentID: import.meta.env.VITE_FLAGSMITH_ENV_ID,
              api: import.meta.env.VITE_FLAGSMITH_API_HOST,
            }}
            flagsmith={flagsmith}
          >
            <AuthProvider>
              <ProfileProvider>
                <WaitlistProvider>
                  <ExternalLoggingInitializer>
                    <TooltipProvider>
                      <Router />
                      <Toaster />
                    </TooltipProvider>
                  </ExternalLoggingInitializer>
                </WaitlistProvider>
              </ProfileProvider>
            </AuthProvider>
          </FlagsmithProvider>
        </HelmetProvider>
      </QueryClientProvider>
    </ErrorBoundary>
  );
}

export default App;
